SystemOrganization addCategory: #Everett!
SystemOrganization addCategory: #'Everett-Pong'!
SystemOrganization addCategory: #'Everett-Tests'!
SystemOrganization addCategory: #'Everett-TicTacToe'!

TestCase subclass: #EntityManagerTestCase
	instanceVariableNames: 'em'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

!EntityManagerTestCase methodsFor: 'running' stamp: 'EricShahkarami 7/2/2013 11:47'!
setUp 

	em := EntityManager new.! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 18:50'!
testEntityMatches

	| e1 e2 pc1 bc1 bc2 c1 c2 c3 s1 s2 |
	
	e1 := em createEntity.
	e2 := em createEntity.

	pc1 := PositionComponent position: 3@7.
	bc1 := VelocityComponent velocity: 1@1.
	bc2 := VelocityComponent velocity: 2@0.
	
	em addComponent: pc1 toEntity: e1.
	em addComponent: bc1 toEntity: e1.
	em addComponent: bc2 toEntity: e2.
	
	c1 := Set newFrom: #(#position).
	c2 := Set newFrom: #(#velocity).
	c3 := Set newFrom: #(#position #velocity).

	s1 := Set new.
	s2 := Set new.
	s1 add: e1.
	s2 add: e1.
	s2 add: e2.
	
	self assert: ( (em entitiesContainingComponents: c1) = s1 ).
	self assert: ( (em entitiesContainingComponents: c2) = s2 ).
	self assert: ( (em entitiesContainingComponents: c3) = s1 )! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/6/2013 23:57'!
testRemoveComponent

	| e pc vc c1 c2 s |
	
	e := em createEntity.

	pc := PositionComponent position: 3@7.
	vc := VelocityComponent velocity: 1@1.
	
	em addComponent: pc toEntity: e.
	em addComponent: vc toEntity: e.
	em removeComponent: (PositionComponent identity) fromEntity: e.
		
	c1 := Set newFrom: #(#velocity).
	c2:= Set newFrom: #(#position velocity).
	
	s := Set new.
	s add: e.
	
	self assert: ( (em entitiesContainingComponents: c1) = s ).
	self assert: ( (em entitiesContainingComponents: c2) isEmpty = true ).! !

TestCase subclass: #GroupManagerTestCase
	instanceVariableNames: 'em e1 e2 e3 pc1 bc1 bc2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!GroupManagerTestCase methodsFor: 'running' stamp: 'ShruthiSudhanva 7/12/2013 13:37'!
setUp 

	em := EntityManager new.
	e1 := em createEntity.
	e2 := em createEntity.
	e3 := em createEntity.
	
	pc1 := PositionComponent position: 3@7.
	bc1 := VelocityComponent velocity: 1@1.
	bc2 := VelocityComponent velocity: 2@0.! !

!GroupManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 19:50'!
testAddComponent

	| gm entities |
	
	gm := em groupManager.
	
	em addComponent: pc1 toEntity: e2.
	gm createGroup: #group1 withEntities: (Set newFrom: {e1 . e2}).
	gm createGroup: #group2 withEntities: (Set newFrom: {e1 . e3 }).
	
	gm addComponent: bc2 toGroup: #group1.
	gm addComponent: bc1 toGroup: #group2.
	gm addEntity: e2 toGroup: #group2.

	self assert: (gm entitiesInGroup: #group1) = (Set newFrom:{0 .1}).
	entities := gm entitiesInGroup: #group2 select: [:entity| (em doesEntity:entity containComponents: (Set newFrom: { #position })) = true ].
	self assert: entities = (Set newFrom: { 1 }).
	
	
	
	
! !

!GroupManagerTestCase methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 19:50'!
testRemoveComponent
	
	| gm entities |
	
	gm := em groupManager.
	
	em addComponent: pc1 toEntity: e2.
	gm createGroup: #group1 withEntities: (Set newFrom: {e1 . e2}).
	gm createGroup: #group2 withEntities: (Set newFrom: {e1 . e3 }).
	
	gm addComponent: bc2 toGroup: #group1.
	gm addComponent: bc1 toGroup: #group2.
	gm removeComponent: (bc2 class identity) fromGroup: #group1.

	entities := gm entitiesInGroup: #group1 select: [ :entity | (em doesEntity: entity containComponents: (Set newFrom: { #velocity } )) = true ].
	self assert: entities isEmpty.
	
	gm removeEntity: 0 fromGroup: #group2.
	entities := gm entitiesInGroup: #group2.
	
	self assert: entities = (Set newFrom:{2}).
	
	gm removeGroup: #group1.
	self assert: (gm groups includes: #group1) = false .
	! !

TestCase subclass: #SystemManagerTestCase
	instanceVariableNames: 'em sm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:51'!
setUp

	em := EntityManager new.
	sm := SystemManager new! !

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/7/2013 15:21'!
testSystemUpdate

	"Tests that e1 and e2 have their positions updated but not e3, which doesn't have a velocity component"

	| e1 e2 e3 pc1 pc2 pc3 vc1 vc2 |
	
	e1 := em createEntity.
	e2 := em createEntity.
	e3 := em createEntity.

	pc1 := PositionComponent position: 3@4.
	pc2 := PositionComponent position: 1@1.
	pc3 := PositionComponent position: 9@9.
	vc1 := VelocityComponent velocity: 1@1.
	vc2 := VelocityComponent velocity: 2@0.

	em addComponent: pc1 toEntity: e1.
	em addComponent: pc2 toEntity: e2.
	em addComponent: pc3 toEntity: e3.
	em addComponent: vc1 toEntity: e1.
	em addComponent: vc2 toEntity: e2.
	
	sm addSystem: (MovementSystem new).
	sm updateEntitiesOf: em afterDeltaTime: 1.
	
	self assert: ( (pc1 position) = (4@5) ).
	self assert: ( (pc2 position) = (3@1) ).
	self assert: ( (pc3 position) = (9@9) )! !

RectangleMorph subclass: #BatMorph
	instanceVariableNames: 'hasMovedUp hasMovedDown hasMoved'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!BatMorph methodsFor: 'events-processing' stamp: 'eshkrm 7/16/2013 18:00'!
handleListenEvent: anEvent

	(anEvent isKeyboard) ifTrue: [ self keyDown: anEvent ]! !

!BatMorph methodsFor: 'accessing' stamp: 'eshkrm 7/18/2013 13:58'!
hasMoved

	^hasMoved! !

!BatMorph methodsFor: 'accessing' stamp: 'eshkrm 7/18/2013 14:00'!
hasMoved: aNumber 

	hasMoved := aNumber! !

!BatMorph methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 17:50'!
hasMovedDown

	^hasMovedDown! !

!BatMorph methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 17:49'!
hasMovedUp

	^hasMovedUp! !

!BatMorph methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 15:38'!
initialize

	super initialize.
	self width: 15.
	self height: 75.
	self color: (Color white).
	self hasMoved: 0! !

!BatMorph methodsFor: 'event handling' stamp: 'eshkrm 7/18/2013 13:58'!
keyDown: anEvent

	| keyValue |

	keyValue := anEvent keyValue.
	(keyValue = 30) ifTrue: [ hasMoved := -1 ].
	(keyValue = 31) ifTrue: [ hasMoved := 1 ]! !

SimpleButtonMorph subclass: #CellMorph
	instanceVariableNames: 'wasClicked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!CellMorph methodsFor: 'button' stamp: 'eshkrm 7/9/2013 16:26'!
doButtonAction 

	wasClicked := true.
	self color: (Color orange)! !

!CellMorph methodsFor: 'initialize-release' stamp: 'eshkrm 7/9/2013 16:25'!
initialize 

	super initialize.
	self label: ''.
	self borderWidth: 2.
	bounds := 0@0 corner: 64@64.
	self useSquareCorners.
	wasClicked := false
	! !

!CellMorph methodsFor: 'event handling' stamp: 'eshkrm 7/9/2013 16:26'!
mouseDown: anEvent

	oldColor := self fillStyle. 
	actWhen == #buttonDown
		ifTrue: [ self doButtonAction]! !

!CellMorph methodsFor: 'accessing' stamp: 'eshkrm 7/9/2013 21:08'!
wasClicked

	^wasClicked ! !

BorderedMorph subclass: #PongWindow
	instanceVariableNames: 'listeners'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongWindow methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/18/2013 13:52'!
addEventListener: anObject

	listeners add: anObject! !

!PongWindow methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 01:04'!
initialize

	super initialize.
	listeners := Set new.
	self color: (Color white).
	self bounds: (0 @ 0 extent: (512) @ (512) + (2 * self borderWidth)).
	self openInWorld.! !

!PongWindow methodsFor: 'event handling' stamp: 'eshkrm 7/18/2013 01:08'!
mouseEnter: anEvent

	listeners do: [ :listener | ActiveHand addEventListener: listener ].
	"anEvent hand newKeyboardFocus: self"! !

!PongWindow methodsFor: 'event handling' stamp: 'eshkrm 7/18/2013 01:08'!
mouseLeave: anEvent

	listeners do: [ :listener | ActiveHand removeEventListener: listener ].! !

Object subclass: #EntityFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityFactory commentStamp: '<historical>' prior: 0!
I am a factory that creates entities with some specified component instances. Subclass me to specify which components.!

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/8/2013 14:34'!
createEntity: anEntityManager 

	| entity components |
	
	entity := anEntityManager createEntity.
	components := self obtainComponents.
	components do: [ :component | anEntityManager addComponent: component toEntity: entity ].
	^entity! !

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/11/2013 19:04'!
createInactiveEntity: anEntityManager

	| entity |
	
	entity := self createEntity: anEntityManager.
	anEntityManager deactivateEntity: entity.
	^entity! !

!EntityFactory class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/8/2013 14:40'!
obtainComponents
	
	^self subclassResponsibility ! !

Object subclass: #EntityManager
	instanceVariableNames: 'entityTable nextAvailableEntity changedEntities inactiveEntities removedEntities groupManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityManager commentStamp: '<historical>' prior: 0!
I am responsible for creating, removing and tracking all entities and their components.
!

!EntityManager methodsFor: 'classify' stamp: 'eshkrm 7/7/2013 13:57'!
activateEntity: anEntity 

	inactiveEntities remove: anEntity.
	self changeEntity: anEntity! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:25'!
addComponent: aComponentInstance toEntity: anEntity

	(anEntity < nextAvailableEntity and: anEntity >= 0) ifTrue: 
	[  
		self entityAt: anEntity putComponent: aComponentInstance.
		self changeEntity: anEntity
	]! !

!EntityManager methodsFor: 'classify' stamp: 'eshkrm 7/6/2013 23:54'!
changeEntity: anEntity

	((changedEntities occurrencesOf: anEntity) > 1) ifFalse: [ changedEntities add: anEntity ]! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 19:34'!
changedEntitiesDo: aBlock

	(changedEntities asSet) do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:39'!
clearChangedEntities

	self changedEntitiesDo: [ :entity | changedEntities remove: entity ]! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:39'!
clearRemovedEntities

	self removedEntitiesDo: [ :entity | removedEntities remove: entity ]! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 09:38'!
component: aComponentIdentifier fromEntity: anEntity

	| componentsOfEntity |
	
	componentsOfEntity := self entityAt: anEntity.
	^(componentsOfEntity at: aComponentIdentifier)! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:27'!
createEntity

	entityTable at: nextAvailableEntity put: (Dictionary new).	
	self changeEntity: nextAvailableEntity.
	nextAvailableEntity := nextAvailableEntity + 1.
	^(nextAvailableEntity - 1)
! !

!EntityManager methodsFor: 'classify' stamp: 'eshkrm 7/7/2013 13:49'!
deactivateEntity: anEntity

	inactiveEntities add: anEntity! !

!EntityManager methodsFor: 'testing' stamp: 'eshkrm 6/13/2013 19:41'!
doesEntity: anEntity containComponent: aComponentIdentifier

	| entityComponents |
	
	entityComponents := (self entityAt: anEntity) keys asSet.
	^(entityComponents includes: aComponentIdentifier)! !

!EntityManager methodsFor: 'testing' stamp: 'eshkrm 6/13/2013 18:34'!
doesEntity: anEntity containComponents: aSetOfComponentIdentifiers

	| entityComponents |
	
	entityComponents := (self entityAt: anEntity) keys asSet.
	^( (aSetOfComponentIdentifiers intersection: entityComponents) = aSetOfComponentIdentifiers )! !

!EntityManager methodsFor: 'accessing' stamp: 'eshkrm 7/6/2013 18:50'!
entitiesContainingComponents: aSetOfComponents

	^( self entitiesSelect: [ :entity | self doesEntity: entity containComponents: aSetOfComponents ] )! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 23:51'!
entitiesDo: aBlock

	entityTable keysDo: [ :entity | aBlock value: entity ].! !

!EntityManager methodsFor: 'enumerate' stamp: 'EricShahkarami 7/2/2013 11:22'!
entitiesSelect: aBlock

	| selectEntities |
	
	selectEntities := Set new.
	entityTable keysDo: [ :entity | (aBlock value: entity) ifTrue: [ selectEntities add: entity ]  ].
	^selectEntities! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity
	
	^(entityTable at: anEntity)! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity putComponent: aComponentInstance 
		
	(entityTable at: anEntity) at: (aComponentInstance class identity) put: aComponentInstance. ! !

!EntityManager methodsFor: 'accessing' stamp: 'ShruthiSudhanva 7/8/2013 17:07'!
groupManager

	^groupManager.! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 13:53'!
inactiveEntitiesDo: aBlock 

	inactiveEntities do: [ :entity | aBlock value: entity ]! !

!EntityManager methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 19:49'!
initialize 
	
	super initialize.
	entityTable := Dictionary new.
	nextAvailableEntity := 0.
	changedEntities := Bag new.
	inactiveEntities := Set new.
	removedEntities := Set new.
	groupManager := GroupManager entityManager: self.! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/6/2013 19:25'!
removeComponent: aComponentIdentifier fromEntity: anEntity

	| components |
	
	components := self entityAt: anEntity.
	components removeKey: aComponentIdentifier.
	self changeEntity: anEntity
! !

!EntityManager methodsFor: 'add-remove' stamp: 'eshkrm 7/11/2013 19:59'!
removeEntity: anEntity	

	"Removes entity from entityTable and places entity in removed pile."	
	
	entityTable removeKey: anEntity.
	removedEntities add: anEntity
	! !

!EntityManager methodsFor: 'enumerate' stamp: 'eshkrm 7/6/2013 19:15'!
removedEntitiesDo: aBlock 

	removedEntities do: [ :entity | aBlock value: entity ]! !

Object subclass: #EverettComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettComponent commentStamp: '<historical>' prior: 0!
I hold data for entities. Systems modify me.!

EverettComponent subclass: #BallMorphComponent
	instanceVariableNames: 'window morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!BallMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 6/13/2013 18:59'!
addMorphToWindow: aWindow

	^(self new addMorphToWindow: aWindow)! !

!BallMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 6/13/2013 18:29'!
identity

	^#ball! !

!BallMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 6/13/2013 18:58'!
addMorphToWindow: aWindow

	window := aWindow.
	window addMorph: morph.! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:54'!
color

	^(morph color)! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:54'!
color: aColor

	morph color: aColor! !

!BallMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 6/13/2013 18:57'!
initialize 

	super initialize.
	morph := BallMorph new.! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:54'!
position

	^(morph position)! !

!BallMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 18:55'!
position: aPosition

	morph position: aPosition! !

EverettComponent subclass: #BatMorphComponent
	instanceVariableNames: 'morph window'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!BatMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 6/13/2013 19:10'!
addMorphToWindow: aWindow 

	^(self new addMorphToWindow: aWindow) ! !

!BatMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 6/13/2013 18:29'!
identity 

	^#bat! !

!BatMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 13:53'!
addMorphToWindow: aWindow

	window := aWindow.
	window addMorph: morph! !

!BatMorphComponent methodsFor: 'events-processing' stamp: 'eshkrm 7/18/2013 13:53'!
addToWindowEventListeners

	window addEventListener: morph.! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/17/2013 13:30'!
color

	^(morph color)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/17/2013 13:30'!
color: aColor

	morph color: aColor! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/18/2013 15:39'!
hasMoved

	^(morph hasMoved)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/18/2013 15:41'!
hasMoved: aNumber

	morph hasMoved: aNumber! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 17:54'!
hasMovedDown

	^(morph hasMovedDown)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 6/13/2013 17:54'!
hasMovedUp

	^(morph hasMovedUp)! !

!BatMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 6/13/2013 19:13'!
initialize 

	super initialize.
	morph := BatMorph new! !

!BatMorphComponent methodsFor: 'display' stamp: 'eshkrm 7/16/2013 21:25'!
openInWorld

	morph openInWorld! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 21:00'!
position

	^(morph position)! !

!BatMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 21:01'!
position: aPosition 

	morph position: aPosition ! !

EverettComponent subclass: #BlueComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!BlueComponent class methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/17/2013 17:39'!
identity 
	^#blue.! !

!BlueComponent methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/17/2013 17:50'!
addBlueComponent: anEntity
	^anEntity color: (Color blue)! !

!BlueComponent methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/17/2013 21:05'!
color
	^ Color blue.! !

EverettComponent subclass: #BoardIndexComponent
	instanceVariableNames: 'boardIndex boardindex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!BoardIndexComponent class methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/17/2013 17:32'!
identity
	^#boardIndex! !

!BoardIndexComponent methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/17/2013 17:20'!
boardIndex
	
	^boardIndex! !

!BoardIndexComponent methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/17/2013 17:20'!
boardIndex: anIndex
	boardIndex := anIndex.! !

EverettComponent subclass: #CellMorphComponent
	instanceVariableNames: 'morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!CellMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 17:54'!
identity

	^#cell! !

!CellMorphComponent class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/9/2013 15:20'!
position: aPosition 

	^(self new position: aPosition)! !

!CellMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 15:00'!
color: aColor

	morph color: aColor! !

!CellMorphComponent methodsFor: 'initialize-release' stamp: 'eshkrm 7/16/2013 15:25'!
initialize

	morph := CellMorph new.
! !

!CellMorphComponent methodsFor: 'accessing' stamp: 'eshkrm 7/16/2013 14:59'!
morph

	^morph! !

!CellMorphComponent methodsFor: 'geometry' stamp: 'eshkrm 7/16/2013 14:59'!
position: aPosition

	morph position: aPosition! !

!CellMorphComponent methodsFor: 'testing' stamp: 'eshkrm 7/16/2013 14:59'!
wasClicked

	^(morph wasClicked)! !

!EverettComponent class methodsFor: 'instance creation' stamp: 'eshkrm 7/14/2013 15:21'!
identity

	^self subclassResponsibility! !

EverettComponent subclass: #NewlyCapturedComponent
	instanceVariableNames: 'entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!NewlyCapturedComponent class methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 01:19'!
identity
 	^#newlyAdded! !

EverettComponent subclass: #PositionComponent
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!PositionComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the position of an entity.!

!PositionComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:09'!
identity
	^#position! !

!PositionComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
position: aPosition

	^(self new position: aPosition)! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
position

	^position! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:02'!
position: aPosition 
	
	position := aPosition! !

EverettComponent subclass: #RedComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!RedComponent class methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/17/2013 17:39'!
identity 		
	^#red! !

!RedComponent methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/17/2013 17:50'!
addRedComponent: anEntity
	^anEntity color: (Color red)! !

!RedComponent methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/17/2013 21:05'!
color
	^Color red.! !

EverettComponent subclass: #VelocityComponent
	instanceVariableNames: 'velocity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!VelocityComponent commentStamp: '<historical>' prior: 0!
I am a component that holds the velocity of an entity.!

!VelocityComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:27'!
identity 	
	^#velocity! !

!VelocityComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
velocity: aVelocity

	^(self new velocity: aVelocity)! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity

	^velocity ! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity: aVelocity
	
	velocity := aVelocity! !

Object subclass: #EverettSystem
	instanceVariableNames: 'updateInterval lastProcessed interestedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettSystem commentStamp: '<historical>' prior: 0!
I am a system that processes entities.
!

!EverettSystem class methodsFor: 'instance creation' stamp: 'eshkrm 7/14/2013 15:20'!
identity

	^self subclassResponsibility ! !

!EverettSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/14/2013 15:10'!
initialize

	updateInterval := 0.
	lastProcessed := 0.
	interestedEntities := Set new! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 15:17'!
interestedEntities

	^interestedEntities! !

!EverettSystem methodsFor: 'accessing' stamp: 'eshkrm 7/11/2013 19:52'!
lastProcessed 

	^lastProcessed! !

!EverettSystem methodsFor: 'accessing' stamp: 'eshkrm 7/11/2013 18:46'!
lastProcessed: aTime

	lastProcessed := aTime! !

!EverettSystem methodsFor: 'processing' stamp: 'eshkrm 7/14/2013 15:11'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt
	
	interestedEntities do: [ :entity | self updateEntity: entity from: anEntityManager afterDeltaTime: dt ].
	! !

!EverettSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 11:12'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	^self subclassResponsibility! !

!EverettSystem methodsFor: 'accessing' stamp: 'eshkrm 7/11/2013 19:52'!
updateInterval

	^updateInterval! !

!EverettSystem methodsFor: 'accessing' stamp: 'AnkuAdhikari 7/11/2013 15:59'!
updateInterval: anUpdateInterval
	
	updateInterval := anUpdateInterval ! !

!EverettSystem methodsFor: 'testing' stamp: 'eshkrm 7/14/2013 15:20'!
verifyEntity: anEntity from: anEntityManager

	^self subclassResponsibility
! !

EverettSystem subclass: #InputHandlerSystem
	instanceVariableNames: 'turn newlyCapturedComponent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!InputHandlerSystem class methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/12/2013 15:26'!
identity
	^#inputHandler! !

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 01:19'!
initialize
	
	super initialize.
	turn := #red.
	newlyCapturedComponent := NewlyCapturedComponent new.! !

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 01:16'!
newTurn
	(turn = #red) ifTrue: [ ^#blue ] ifFalse:[^#red].! !

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 21:20'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt
"click the same cell again - needs to be handled"
	| unclickedCells clickedCell|

	unclickedCells := anEntityManager entitiesSelect: [:entity | (self verifyEntity: entity from: anEntityManager) = false ].
	
	clickedCell := unclickedCells detect:[:cell | (anEntityManager component: (CellMorphComponent identity) fromEntity: cell) wasClicked = true] ifNone:[nil].
	
	clickedCell ifNotNil: [ 
	self updateEntity: clickedCell from: anEntityManager  afterDeltaTime: dt.
	turn := self newTurn]! !

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 11:56'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	| cellMorphComponent colorComponent|
	
	cellMorphComponent := anEntityManager component: (CellMorphComponent identity) fromEntity: anEntity.
	
	(turn = #red) 
	ifTrue: [ colorComponent := RedComponent new ] 
	ifFalse:[ colorComponent := BlueComponent new].
	anEntityManager addComponent: colorComponent  toEntity: anEntity.
	
	cellMorphComponent color: (colorComponent color).
	
	anEntityManager addComponent: newlyCapturedComponent  toEntity: anEntity.! !

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 21:13'!
verifyEntity: anEntity from: anEntityManager 

	| containsCellMorphComponent containsRedComponent containsBlueComponent |
	
	containsCellMorphComponent := anEntityManager doesEntity: anEntity  containComponent: CellMorphComponent identity.
	containsRedComponent := anEntityManager doesEntity: anEntity  containComponent: RedComponent identity.
	containsBlueComponent := anEntityManager doesEntity: anEntity  containComponent: BlueComponent identity.
	
	^(containsCellMorphComponent & (containsRedComponent | containsBlueComponent )).! !

!InputHandlerSystem methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/17/2013 19:16'!
whoseTurn
	|Red Blue|
	((self turn)= Red) ifTrue: [self turn= Blue] ifFalse: [self turn=Red].
	^turn
	! !

EverettSystem subclass: #MovementSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!MovementSystem commentStamp: '<historical>' prior: 0!
I am a system that updates the position of entities.!

!MovementSystem class methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:55'!
identity 

	^#movement! !

!MovementSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 15:56'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| pc vc |
	
	pc := anEntityManager component: (PositionComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	pc position: ( pc position + (dt * (vc velocity)) )! !

!MovementSystem methodsFor: 'initialize-release' stamp: 'eshkrm 7/14/2013 15:20'!
verifyEntity: anEntity from: anEntityManager

	| interestedComponents |
	
	interestedComponents := { PositionComponent identity. VelocityComponent identity. } asSet.
	^(anEntityManager doesEntity: anEntity containComponents: interestedComponents) ! !

EverettSystem subclass: #PongCollisionSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongCollisionSystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/18/2013 13:25'!
identity

	^#collision! !

!PongCollisionSystem methodsFor: 'private' stamp: 'eshkrm 7/18/2013 13:31'!
ballEntity

	! !

!PongCollisionSystem methodsFor: 'processing' stamp: 'eshkrm 7/18/2013 13:26'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt! !

!PongCollisionSystem methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 13:25'!
verifyEntity: anEntity from: anEntityManager

	| isValidEntity |
	
	isValidEntity := anEntityManager doesEntity: anEntity containComponent: (BatMorphComponent identity).
	isValidEntity := isValidEntity or: (anEntityManager doesEntity: anEntity containComponent: BallMorphComponent identity).
	isValidEntity := isValidEntity and: (anEntityManager doesEntity: anEntity containComponent: VelocityComponent identity).
		
	^isValidEntity! !

EverettSystem subclass: #PongInputSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongInputSystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/18/2013 00:50'!
identity 

	^#input! !

!PongInputSystem methodsFor: 'processing' stamp: 'eshkrm 7/18/2013 21:12'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	| mc vc |
	
	mc := anEntityManager component: (BatMorphComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.
	vc velocity: (5 * (0@(mc hasMoved)))! !

!PongInputSystem methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 20:59'!
verifyEntity: anEntity from: anEntityManager

	| gm |
		
	gm := anEntityManager groupManager.	
	^(gm doesGroup: #player includeEntity: anEntity)
! !

EverettSystem subclass: #PongMovementSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongMovementSystem class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/17/2013 16:22'!
identity 

	#movement! !

!PongMovementSystem methodsFor: 'private' stamp: 'eshkrm 6/13/2013 19:50'!
morphComponentFromEntity: anEntity in: anEntityManager

	(anEntityManager doesEntity: anEntity containComponent: (BallMorphComponent identity))
		ifTrue: [ ^(anEntityManager component: (BallMorphComponent identity) fromEntity: anEntity) ]
		ifFalse: [ ^(anEntityManager component: (BatMorphComponent identity) fromEntity: anEntity) ]! !

!PongMovementSystem methodsFor: 'processing' stamp: 'eshkrm 7/18/2013 21:13'!
updateBallEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| mc vc |
		
	mc := anEntityManager component: (BallMorphComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	mc position: ( mc position + (dt * (vc velocity)) ).! !

!PongMovementSystem methodsFor: 'processing' stamp: 'eshkrm 7/18/2013 21:43'!
updateBatEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| mc vc |
		
	mc := anEntityManager component: (BatMorphComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	mc position: ( mc position + (dt * (vc velocity)) ).
	vc velocity: (self updateBatVelocity: (vc velocity)).
	mc hasMoved: 0.! !

!PongMovementSystem methodsFor: 'private' stamp: 'eshkrm 7/18/2013 21:44'!
updateBatVelocity: aVelocity

	| norm |
	
	norm := (aVelocity dist: 0@0) abs.
	(norm < 1) ifTrue: [ ^(0@0) ] ifFalse: [ ^(0.95 * aVelocity) ]  ! !

!PongMovementSystem methodsFor: 'processing' stamp: 'eshkrm 7/18/2013 21:10'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	| gm |

	gm := anEntityManager groupManager.
	gm entitiesInGroup: #bats do: [ :entity | self updateBatEntity: entity from: anEntityManager afterDeltaTime: dt ].
	gm entitiesInGroup: #ball do: [ :entity | self updateBallEntity: entity from: anEntityManager afterDeltaTime: dt ].! !

!PongMovementSystem methodsFor: 'processing' stamp: 'eshkrm 6/13/2013 19:51'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt 

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| mc vc |
		
	mc := self morphComponentFromEntity: anEntity in: anEntityManager. 
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	mc position: ( mc position + (dt * (vc velocity)) )! !

!PongMovementSystem methodsFor: 'testing' stamp: 'eshkrm 6/13/2013 19:43'!
verifyEntity: anEntity from: anEntityManager

	| isValidEntity |
	
	isValidEntity := anEntityManager doesEntity: anEntity containComponent: (BatMorphComponent identity).
	isValidEntity := isValidEntity or: (anEntityManager doesEntity: anEntity containComponent: BallMorphComponent identity).
	isValidEntity := isValidEntity and: (anEntityManager doesEntity: anEntity containComponent: VelocityComponent identity).
		
	^isValidEntity
	
! !

EverettSystem subclass: #VictorySystem
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!VictorySystem class methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 11:28'!
identity 
	^#victory.! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/19/2013 14:43'!
checkGroupStrike: aGroup 

			
	((GroupManager doesGroup:  aGroup containComponent: #red) )ifTrue:[^true] .
	((GroupManager doesGroup:  aGroup containComponent: #blue) )ifTrue:[^true] .
	^false
	
	! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/19/2013 14:44'!
checkStrike
	|grpset |
	grpset:= Set new.
	grpset:= #( #row1 #row2 #row3 #col1 #col2 #col3 #diag1 #diag2).
	grpset do: [ :group | (self checkGroupStrike: group) ifTrue: [ ^true] ].
	^false
	
	
	
	! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/19/2013 14:56'!
clearGame

	"This should reset the game by clearing all values"
	
	
	
	! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 11:28'!
initialize
	super initialize! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 14:13'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	| anEntity |
	
	anEntity := anEntityManager entitiesContainingComponents: (NewlyCapturedComponent identity).
	anEntityManager removeComponent:  (NewlyCapturedComponent identity) fromEntity: anEntity.
	
	self updateEntity: anEntity  from: anEntityManager  afterDeltaTime: dt.! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 16:46'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	
	! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/18/2013 16:52'!
verifyEntity: anEntity from: anEntityManager 

		
	^(anEntityManager doesEntity: anEntity containComponent: NewlyCapturedComponent identity).
	
	! !

!VictorySystem methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/19/2013 15:08'!
winDeclare
	|winner|
	winner:=NewlyCapturedComponent identity.
	(self checkStrike) ifTrue:[
		(winner=#red) ifTrue: ['Game Won by Player Red!!!!!!!!' asMorph openInWorld] ifFalse: ['Game Won by Player Blue!!!!!!!!' asMorph openInWorld].
		self clearGame.] 
		ifFalse: []
	
	
	
	
	
	
	
	
	
	
	! !

Object subclass: #GroupManager
	instanceVariableNames: 'groups entityManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!GroupManager commentStamp: '<historical>' prior: 0!
I manage groups of entities, each identified with a tag.!

!GroupManager class methodsFor: 'as yet unclassified' stamp: 'eshkrm 7/14/2013 15:21'!
entityManager: anEntityManager

	^self new entityManager: anEntityManager.! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/14/2013 14:53'!
activateEntitiesInGroup: aGroupIdentifier

	self entitiesInGroup: aGroupIdentifier do: [ :entity | entityManager activateEntity: entity ]! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/14/2013 14:51'!
addComponent: aComponentInstance toGroup: aGroupIdentifier

	(self entitiesInGroup: aGroupIdentifier) do: [ :entity | entityManager addComponent: aComponentInstance toEntity: entity ]! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/8/2013 23:00'!
addEntity: anEntity toGroup: aGroupIdentifier

	(groups at: aGroupIdentifier) add: anEntity! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/8/2013 22:55'!
createGroup: aGroupIdentifier

	groups at: aGroupIdentifier put: Set new! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/14/2013 14:53'!
createGroup: aGroupIdentifier withEntities: aCollectionOfEntities

	groups at: aGroupIdentifier put: (aCollectionOfEntities asSet)! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/14/2013 14:54'!
deactivateEntitiesInGroup: aGroupIdentifier

	self entitiesInGroup: aGroupIdentifier do: [ :entity | entityManager deactivateEntity: entity ]! !

!GroupManager methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/18/2013 16:25'!
doesGroup: aGroup containComponent: aComponentIdentifier
	
	^((self entitiesInGroup: aGroup detect: [:entity | (entityManager doesEntity: entity containComponent: aComponentIdentifier) = false  ]) isNil).
	
	! !

!GroupManager methodsFor: 'testing' stamp: 'eshkrm 7/18/2013 20:57'!
doesGroup: aGroup includeEntity: anEntity

	^( (self entitiesInGroup: aGroup) includes: anEntity )! !

!GroupManager methodsFor: 'accessing' stamp: 'eshkrm 7/14/2013 14:57'!
entitiesInGroup: aGroupIdentifier

	^(groups at: aGroupIdentifier)! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier collect: aBlock

	^( (groups at: aGroupIdentifier) collect: [ :entity | aBlock value: entity ] )! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier detect: aBlock

	^( (groups at: aGroupIdentifier) detect: [ :entity | aBlock value: entity ] ).	! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:01'!
entitiesInGroup: aGroupIdentifier do: aBlock

	(groups at: aGroupIdentifier) do: [ :entity | aBlock value: entity ]! !

!GroupManager methodsFor: 'enumerate' stamp: 'eshkrm 7/8/2013 23:02'!
entitiesInGroup: aGroupIdentifier select: aBlock

	^( (groups at: aGroupIdentifier) select: [ :entity | aBlock value: entity ] )! !

!GroupManager methodsFor: 'initialize-release' stamp: 'ShruthiSudhanva 7/12/2013 12:11'!
entityManager: anEntityManager 	

	super initialize.
	groups := Dictionary new.
	entityManager := anEntityManager.! !

!GroupManager methodsFor: 'accessing' stamp: 'eshkrm 7/14/2013 14:51'!
groups

	^(groups keys).! !

!GroupManager methodsFor: 'add-remove' stamp: 'eshkrm 7/14/2013 14:51'!
removeComponent: aComponentIdentifier fromGroup: aGroupIdentifier

	(self entitiesInGroup: aGroupIdentifier) do: [ :entity | entityManager removeComponent: aComponentIdentifier fromEntity: entity ]
! !

!GroupManager methodsFor: 'add-remove' stamp: 'ShruthiSudhanva 7/7/2013 20:15'!
removeEntity: anEntity fromGroup: aGroupIdentfier

	(groups at: aGroupIdentfier) remove: anEntity. 	! !

!GroupManager methodsFor: 'modify-groups' stamp: 'eshkrm 7/8/2013 22:55'!
removeGroup: aGroupIdentifier

	groups removeKey: aGroupIdentifier
! !

Object subclass: #NameOfSubclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

Object subclass: #PongGame
	instanceVariableNames: 'window em sm prevUpdateTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!PongGame methodsFor: 'gameplay' stamp: 'eshkrm 6/13/2013 17:36'!
calculateDeltaTime

	| currUpdateTime prevMilliseconds currMilliseconds secondsDt millisecondsDt |
	
	currUpdateTime := Time now.
	
	prevMilliseconds := (prevUpdateTime nanoSecond) / 1000000000.
	currMilliseconds := (currUpdateTime nanoSecond) / 1000000000.
	(currMilliseconds < prevMilliseconds) ifTrue: [ currMilliseconds := currMilliseconds + 1 ].
	
	millisecondsDt := currMilliseconds - prevMilliseconds.
	secondsDt := (currUpdateTime subtractTime: prevUpdateTime) seconds.
	
	prevUpdateTime := currUpdateTime.
	
	^(secondsDt + millisecondsDt)! !

!PongGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 20:55'!
initialize 

	super initialize.
	window := PongWindow new.
	em := EntityManager new.
	sm := SystemManager new.
	self initializeEntities.
	self initializeSystems.! !

!PongGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 16:51'!
initializeEntities

	| gm bat1 bat2 ball mc1 mc2 mc3 |
	
	gm := em groupManager.
	
	bat1 := em createEntity.
	bat2 := em createEntity.	
	ball := em createEntity.
	
	gm createGroup: #player withEntities: { bat1 }.
	gm createGroup: #opponent withEntities: { bat2 }. 
	gm createGroup: #bats withEntities: { bat1. bat2 }.
	gm createGroup: #ball withEntities: { ball }.
		
	mc1 := BatMorphComponent addMorphToWindow: window.
	mc2 := BatMorphComponent addMorphToWindow: window.
	mc3 := BallMorphComponent addMorphToWindow: window.
	mc1 addToWindowEventListeners.
	mc1 position: 45@60.
	mc2 position: 450@60.
	mc3 position: 256@256.
	
	em addComponent: mc1 toEntity: bat1.
	em addComponent: mc2 toEntity: bat2.
	em addComponent: mc3 toEntity: ball.
	
	em addComponent: (VelocityComponent velocity: 0@0) toEntity: bat1.
	em addComponent: (VelocityComponent velocity: 0@0) toEntity: bat2.
	em addComponent: (VelocityComponent velocity: 0@0) toEntity: ball! !

!PongGame methodsFor: 'initialize-release' stamp: 'eshkrm 7/18/2013 00:49'!
initializeSystems

	sm addSystem: (PongInputSystem new).
	sm addSystem: (PongMovementSystem new).! !

!PongGame methodsFor: 'gameplay' stamp: 'eshkrm 7/18/2013 15:12'!
play

	"| dt |
	
	prevUpdateTime := Time now."
		
	[ [ true ] whileTrue: [ sm updateEntitiesOf: em afterDeltaTime: 1 ] ] forkAt: (Processor activePriority - 1) ! !

Object subclass: #SystemManager
	instanceVariableNames: 'priorities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!SystemManager commentStamp: '<historical>' prior: 0!
I manage all running systems.!

!SystemManager methodsFor: 'add-remove' stamp: 'EricShahkarami 7/1/2013 15:18'!
addSystem: aSystemInstance 
	
	self addSystem: aSystemInstance withPriority: 0 ! !

!SystemManager methodsFor: 'add-remove' stamp: 'eshkrm 7/7/2013 21:01'!
addSystem: aSystemInstance withPriority: aPriority
	
	self priorityAt: aPriority putSystem: aSystemInstance.


	! !

!SystemManager methodsFor: 'processing' stamp: 'eshkrm 7/11/2013 18:56'!
haveSystem: system updateEntitiesFrom: anEntityManager afterDeltaTime: dt

	| lastProcessed updateInterval |
	
	self resolveChangedEntitiesFrom: anEntityManager withSystem: system.
	self resolveInactiveEntitiesFrom: anEntityManager withSystem: system.	
	self resolveRemovedEntitiesFrom: anEntityManager withSystem: system.
		
	lastProcessed := (system lastProcessed) + dt.
	updateInterval := system updateInterval.
	
	(lastProcessed >= updateInterval) ifTrue: 
	[ 
		system updateEntitiesOf: anEntityManager afterDeltaTime: dt.
		system lastProcessed: 0.
	]
	ifFalse:
	[  
		system lastProcessed: lastProcessed.
	]! !

!SystemManager methodsFor: 'initialize-release' stamp: 'eshkrm 7/8/2013 22:39'!
initialize
	
	super initialize.
	priorities := SortedCollection sortBlock: [ :p1 :p2 | p1 key >= p2 key ]! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/7/2013 20:16'!
priorityAt: aPriority

	| systemsWithAPriority |
	
	systemsWithAPriority := priorities detect: [ :assoc | assoc key = aPriority ].
	^(systemsWithAPriority value)! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/7/2013 21:00'!
priorityAt: aPriority putSystem: aSystemInstance

	| systemsWithAPriority |
	
	(priorities includes: aPriority) ifFalse: [ priorities add: ( aPriority->(Dictionary new) ) ].
	
	systemsWithAPriority := self priorityAt: aPriority.
	systemsWithAPriority at: (aSystemInstance class identity) put: aSystemInstance! !

!SystemManager methodsFor: 'add-remove' stamp: 'ShruthiSudhanva 7/7/2013 22:02'!
removeSystem: aSystemIdentifier

	| systems |

	systems := self systemsWithPriorityDetect: [ :systemsWithPriority | systemsWithPriority includesKey: aSystemIdentifier ].
	systems ifNotNil: [ systems removeKey: aSystemIdentifier ]
	! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/14/2013 15:09'!
resolveChangedEntitiesFrom: anEntityManager withSystem: aSystem

	| interestedEntities |
	
	interestedEntities := aSystem interestedEntities.

	anEntityManager changedEntitiesDo: [ :entity | 
		(aSystem verifyEntity: entity from: anEntityManager) 
			ifTrue: [ (interestedEntities includes: entity) ifFalse: [ interestedEntities add: entity ]. ]
			ifFalse: [ (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ].
	]! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/11/2013 18:58'!
resolveInactiveEntitiesFrom: anEntityManager withSystem: aSystem

	| interestedEntities |
	
	interestedEntities := aSystem interestedEntities.
	anEntityManager inactiveEntitiesDo: [ :entity | (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ]! !

!SystemManager methodsFor: 'private' stamp: 'eshkrm 7/11/2013 18:59'!
resolveRemovedEntitiesFrom: anEntityManager withSystem: aSystem 

	| interestedEntities |
	
	interestedEntities := aSystem interestedEntities.
	anEntityManager removedEntitiesDo: [ :entity | (interestedEntities includes: entity) ifTrue: [ interestedEntities remove: entity ] ]! !

!SystemManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 21:23'!
systemsDo: aBlock

	"Run block on systems in order of their priority"
	priorities do: [ :assoc | (assoc value) do: [ :system | aBlock value: system ] ]! !

!SystemManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 21:14'!
systemsWithPriorityDetect: aBlock

	| systemsWithPriority |

	systemsWithPriority := priorities detect: [ :assoc | aBlock value: (assoc value) ].
	^systemsWithPriority ! !

!SystemManager methodsFor: 'enumerate' stamp: 'eshkrm 7/7/2013 21:23'!
systemsWithPriorityDo: aBlock 
	
	"Run block on systems with same priority."
	priorities do: [ :assoc | aBlock value: (assoc value ) ]! !

!SystemManager methodsFor: 'processing' stamp: 'eshkrm 7/11/2013 18:47'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	self systemsDo: [ :system | self haveSystem: system updateEntitiesFrom: anEntityManager afterDeltaTime: dt ].
	
	anEntityManager clearChangedEntities.
	anEntityManager clearRemovedEntities.
	! !

Object subclass: #TicTacToeGame
	instanceVariableNames: 'em window cells sm gm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-TicTacToe'!

!TicTacToeGame methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 21:04'!
addEntity: anEntity toRowGroup: i  toColumnGroup: j

	gm addEntity: anEntity  toGroup: ('row', i asString) ;
	addEntity: anEntity   toGroup: ('col', j asString ) asSymbol.
	
	(i = j) 
	ifTrue:[ gm addEntity: anEntity  toGroup: #diag1].
	
	(((i - j) abs = 2) | (i = 2 & j =2))
	ifTrue:[ gm addEntity: anEntity toGroup: #diag2 ]
			! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'ShruthiSudhanva 7/18/2013 21:06'!
createBoard

	self createGroups.
	1 to: 3 do: [ :i | 1 to: 3 do: [ :j | self createCellAt: i at: j ] ].! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'ShruthiSudhanva 7/18/2013 20:36'!
createCellAt: i at: j

	"Create a cell for position (i,j) and add it to my on-screen representation at the appropriate screen position."
	| b cc e origin bic|
	
	cc := CellMorphComponent new.
	bic := BoardIndexComponent new.
	bic  boardIndex: i@j.
	
	b := cc morph.
	origin := window innerBounds origin.
	window addMorph: b.
	b position: ( (i -1) * b width) @ ((j - 1) * b height ) + origin.
	
 	e := em createEntity.
	em addComponent: cc toEntity: e.
	em addComponent: bic toEntity: e.
	
	self addEntity: e toRowGroup: i toColumnGroup: j.
	
	
	! !

!TicTacToeGame methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 20:43'!
createGroups
	
	gm := em groupManager.
	gm := GroupManager entityManager: em.
	
	1 to: 3 do:[:i | gm createGroup: (('row',i asString ) asSymbol)].
	1 to: 3 do:[:j | gm createGroup: (('col',j asString ) asSymbol)].
	
	gm createGroup: #diag1; createGroup: #diag2.! !

!TicTacToeGame methodsFor: 'initialize-release' stamp: 'ShruthiSudhanva 7/12/2013 15:25'!
initialize

	em := EntityManager new.
	sm := SystemManager new.
	sm addSystem: (InputHandlerSystem new).

	window := BorderedMorph new.
	window bounds: (5 @ 5 extent: (64 * 3) @ (64 * 3) + (2 * window borderWidth)).
	self createBoard! !

!TicTacToeGame methodsFor: 'gameplay' stamp: 'ShruthiSudhanva 7/18/2013 00:17'!
play

	window openInWorld.
	
	[self windowIsOpen] whileTrue: [ sm updateEntitiesOf: em afterDeltaTime: 1 ]
	
	! !

!TicTacToeGame methodsFor: 'as yet unclassified' stamp: 'ShruthiSudhanva 7/18/2013 00:03'!
windowIsOpen
	^World allMorphs includes: window.! !

CircleMorph subclass: #BallMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Pong'!

!BallMorph methodsFor: 'initialize-release' stamp: 'eshkrm 7/17/2013 20:17'!
initialize

	super initialize.
	self color: (Color white).! !
